//
// C++ Implementation: "long long" value spin box widget
//
// Description:
//
//
// Author: Witold Wysota <wysota@wysota.eu.org>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "qwwlongspinbox.h"

#ifndef WW_NO_SPINBOX

#include <QLineEdit>
#include <QMessageBox>
#include <QStyleOptionSpinBox>
#include <QLocale>
#include <QValidator>
#include "wwglobal_p.h"

/**
 *  @class  QwwLongSpinBox
 *  @brief  A spinbox taking long long values
 *
 */


/**
 * \internal
 */
class QwwLongSpinBoxPrivate : public QwwPrivate {
public:
    WW_DECLARE_PUBLIC(QwwLongSpinBox);
    QwwLongSpinBoxPrivate(QwwLongSpinBox *p) : QwwPrivate(p) {
        value = 0;
        minimum = 0;
        maximum = 99;
        singlestep = 1;
    }
    qlonglong value;
    qlonglong minimum;
    qlonglong maximum;
    qlonglong singlestep;
    QString prefix;
    QString suffix;
//     void _q_editorTextChanged(const QString & t);
    void updateEdit();
    void updateButtons();
    QString textFromValue(qlonglong value) const {
        QString s = QLocale().toString(value);
        s.remove(QLocale().groupSeparator());
        return s;
    }
    qlonglong valueFromText(const QString &text) const {
        QString tmp = text;
        if (!prefix.isEmpty() && tmp.startsWith(prefix)) {
            tmp.remove(0, prefix.size());
        }
        if (!suffix.isEmpty() && tmp.endsWith(suffix)) {
            tmp.chop(suffix.size());
        }
        return tmp.toLongLong();
    }
};

/**
 * \internal
 */
class QwwLongSpinBoxValidator : public QValidator {
public:
    QwwLongSpinBoxValidator(QwwLongSpinBox *sb) : QValidator(sb), m_spinbox(sb) {}
    void fixup ( QString & input ) const {
        m_spinbox->fixup(input);
    }
    State validate ( QString & input, int & pos ) const {
        if (!m_spinbox->prefix().isEmpty() && !input.startsWith(m_spinbox->prefix())) {
            input.prepend(m_spinbox->prefix());
        }
        if (!m_spinbox->suffix().isEmpty() && !input.endsWith(m_spinbox->suffix())) {
            input.append(m_spinbox->suffix());
        }
        return m_spinbox->validate(input, pos);
    }
private:
    QwwLongSpinBox *m_spinbox;
};

QwwLongSpinBox::QwwLongSpinBox(QWidget *parent)
        : QAbstractSpinBox(parent), QwwPrivatable(new QwwLongSpinBoxPrivate(this)) {
    Q_D(QwwLongSpinBox);
    d->updateEdit();

    lineEdit()->setValidator(new QwwLongSpinBoxValidator(this));
    connect(lineEdit(), SIGNAL(textChanged(const QString &)), SLOT(_q_editorTextChanged(const QString &)));
}


QwwLongSpinBox::~QwwLongSpinBox() {
}

/**
 * @brief spinbox value
 */
qlonglong QwwLongSpinBox::value() const {
    Q_D(const QwwLongSpinBox);
    return d->value;
}

/**
 * @brief minimum value allowed for the spinbox
 */
qlonglong QwwLongSpinBox::minimum() const {
    Q_D(const QwwLongSpinBox);
    return d->minimum;
}

/**
 * @brief maximum value allowed for the spinbox
 */
qlonglong QwwLongSpinBox::maximum() const {
    Q_D(const QwwLongSpinBox);
    return d->maximum;
}

/**
 * @brief spinbox's prefix
 */
const QString & QwwLongSpinBox::prefix() const {
    Q_D(const QwwLongSpinBox);
    return d->prefix;
}

/**
 * @brief spinbox's suffix
 */
const QString & QwwLongSpinBox::suffix() const {
    Q_D(const QwwLongSpinBox);
    return d->suffix;
}

/**
 *
 * @param value
 * @return
 */
QString QwwLongSpinBox::textFromValue(qlonglong value) const {
    Q_D(const QwwLongSpinBox);
    return d->textFromValue(value);
}

/**
 *
 * @param text
 * @return
 */
qlonglong QwwLongSpinBox::valueFromText(const QString & text) const {
    Q_D(const QwwLongSpinBox);
    return d->valueFromText(text);
}

/**
 *
 * @param v
 */
void QwwLongSpinBox::setMaximum(qlonglong v) {
    if (maximum()==v)
        return;
    Q_D(QwwLongSpinBox);
    setRange(qMin(d->minimum, v), v);
}

/**
 *
 * @param v
 */
void QwwLongSpinBox::setMinimum(qlonglong v) {
    if (minimum()==v)
        return;
    Q_D(QwwLongSpinBox);
    setRange(v, qMax(d->maximum, v));
}

/**
 * @brief Sets the spinbox value
 * @param v
 */
void QwwLongSpinBox::setValue(qlonglong v) {
    v = qBound(minimum(), v, maximum());
    if (value()==v)
        return;
    Q_D(QwwLongSpinBox);
    d->value = v;
    QString textval = d->prefix + textFromValue(v) + d->suffix;
    d->updateEdit();
    emit valueChanged(v);
    emit valueChanged(textval);
}

/**
 * @brief Sets the spinbox prefix
 * @param p
 */
void QwwLongSpinBox::setPrefix(const QString &p) {
    Q_D(QwwLongSpinBox);
    d->prefix = p;
    d->updateEdit();
}

/**
 * @brief Sets the spinbox suffix
 * @param s
 */
void QwwLongSpinBox::setSuffix(const QString &s) {
    Q_D(QwwLongSpinBox);
    d->suffix = s;
    d->updateEdit();
}

/**
 * @brief Returns the step for the spinbox
 * @return
 */
qlonglong QwwLongSpinBox::singleStep() const {
    Q_D(const QwwLongSpinBox);
    return d->singlestep;
}

/**
 * @brief   Sets the step size for the spinbox
 * @param s
 */
void QwwLongSpinBox::setSingleStep(qlonglong s) {
    if (s==0)
        return;
    Q_D(QwwLongSpinBox);
    d->singlestep = s;
}

/**
 * @brief   Sets the range of values allowed for the spinbox
 * @param min
 * @param max
 */
void QwwLongSpinBox::setRange(qlonglong min, qlonglong max) {
    Q_D(QwwLongSpinBox);
    d->minimum = min;
    d->maximum = max;
    if (value()<d->minimum) {
        setValue(d->minimum);
    } else if (value()>d->maximum) {
        setValue(d->maximum);
    }
}


/**
 * @brief   Changes the spinbox value by a number of steps
 * @param steps
 */
void QwwLongSpinBox::stepBy(int steps) {
    if (steps==0)
        return;
    qlonglong nvalue = value()+steps*singleStep();
    if (wrapping()) {
        qlonglong d = nvalue-maximum();
        qlonglong nd = nvalue - minimum();
        if (nvalue>value() && d>0) {
            nvalue = minimum()+d-1;
        } else if (nvalue < value() && nd<0) {
            nvalue = maximum()+nd+1;
        }
    } else {
        nvalue = qBound(minimum(), nvalue, maximum());
    }
    setValue(nvalue);
    selectAll();
}


/**
 * @brief   Validates the spinbox input
 * @param input
 * @param pos
 * @return
 */
QValidator::State QwwLongSpinBox::validate(QString & input, int & pos) const {
    QString tinput = input;
    if (!prefix().isEmpty() && tinput.startsWith(prefix()))
        tinput.remove(0, prefix().size());
    if (!suffix().isEmpty() && tinput.endsWith(suffix()))
        tinput.chop(suffix().size());
    bool ok;
    qlonglong v = tinput.toLongLong(&ok);
    if (!ok)
        return QValidator::Invalid;
    if (v<minimum() || v>maximum())
        return QValidator::Invalid;
    return QValidator::Acceptable;
}

void QwwLongSpinBox::fixup(QString & input) const {
    input.remove(QLocale().groupSeparator());
}


/**
 *
 * @return
 */
QAbstractSpinBox::StepEnabled QwwLongSpinBox::stepEnabled() const {
    if (wrapping())
        return StepUpEnabled|StepDownEnabled;
    StepEnabled se = 0;
    if (value()<maximum()) se |= StepUpEnabled;
    if (value()>minimum()) se |= StepDownEnabled;
    return se;
}

void QwwLongSpinBoxPrivate::updateEdit() {
    Q_Q(QwwLongSpinBox);
    QLineEdit *edit = q->lineEdit();
    QString textval = prefix + q->textFromValue(value) + suffix;
    const bool empty = edit->text().isEmpty();
    int cursor = edit->cursorPosition();
    const bool sb = edit->blockSignals(true);
    edit->setText(textval);
    cursor = qBound(prefix.size(), cursor, edit->displayText().size() - suffix.size());
    edit->setCursorPosition(empty ? prefix.size() : cursor);
    edit->blockSignals(sb);
    updateButtons();
}


void QwwLongSpinBoxPrivate::updateButtons() {
    Q_Q(QwwLongSpinBox);
    QStyleOptionSpinBox opt;
    opt.init(q);
    opt.activeSubControls = 0;
    opt.buttonSymbols = q->buttonSymbols();
    opt.subControls = QStyle::SC_SpinBoxUp | QStyle::SC_SpinBoxDown | QStyle::SC_SpinBoxFrame;
    opt.stepEnabled = q->stepEnabled();
    opt.frame = q->hasFrame();
    q->update(q->style()->subControlRect(QStyle::CC_SpinBox, &opt, QStyle::SC_SpinBoxUp, q));
    q->update(q->style()->subControlRect(QStyle::CC_SpinBox, &opt, QStyle::SC_SpinBoxDown, q));
}

void QwwLongSpinBox/*Private*/::_q_editorTextChanged(const QString & t) {
//     WW_Q(QwwLongSpinBox);
    QString tmp = t;
    int pos = /*q->*/lineEdit()->cursorPosition();
    QValidator::State state = /*q->*/validate(tmp, pos);
    if (state==QValidator::Acceptable) {
        /*q->*/setValue(valueFromText(tmp));
    }
}

#endif // WW_NO_SPINBOX
