//
// C++ Implementation: QwwTipWidget
//
// Description:
//
//
// Author: Witold Wysota <wysota@wysota.eu.org>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "qwwtipwidget.h"
#include <QHBoxLayout>

#include <QCheckBox>
#include <QPushButton>
#include <QTextBrowser>
#include <QVBoxLayout>
#include <QStringListModel>
#include <QEvent>
#include <QPixmap>

/**
 *  \internal
 *  @brief P-impl for QwwTipWidget
 *
 */
class QwwTipWidgetPrivate {
friend class QwwTipWidget;
  QwwTipWidgetPrivate(QwwTipWidget *par){
    pub = par;
    m_headerWidget = 0;
  }
  QwwTipWidget *pub;
  QAbstractItemModel *m_model;
  QWidget *m_headerWidget;
  QPersistentModelIndex m_currentTip;
  QPushButton *m_next, *m_prev, *m_close;
  QTextBrowser *m_browser;
  QCheckBox *m_check;
};

inline QwwTipWidgetPrivate* QwwTipWidget::d_func() { return priv; }
inline QwwTipWidgetPrivate* QwwTipWidget::d_func() const { return priv; }


/**
 *  @class QwwTipWidget
 *  @brief Widget that displays application tips.
 *
 */




/**
 *  @brief initialises widgets and connections
 *
 */
void QwwTipWidget::initUi() {
    Q_D(QwwTipWidget);
    QHBoxLayout *l = new QHBoxLayout;
    d->m_check = new QCheckBox;
    d->m_check->setChecked(true);
    l->addWidget(d->m_check);
    l->addStretch();
    d->m_prev = new QPushButton;
    d->m_prev->setIcon(QPixmap(":/wwwidgets/arrowleft.png"));
    l->addWidget(d->m_prev);
    d->m_next = new QPushButton;
    d->m_next->setIcon(QPixmap(":/wwwidgets/arrowright.png"));
    l->addWidget(d->m_next);
    d->m_close = new QPushButton;
    d->m_close->setDefault(true);
    d->m_close->setAutoDefault(true);
    l->addWidget(d->m_close);
    QVBoxLayout *vl = new QVBoxLayout(this);
    d->m_browser = new QTextBrowser;
    d->m_browser->setOpenExternalLinks(true);
    vl->addWidget(d->m_browser);
    vl->addLayout(l);
    connect(d->m_close, SIGNAL(clicked()), this, SIGNAL(closed()));
    connect(d->m_prev, SIGNAL(clicked()), this, SLOT(prevTip()));
    connect(d->m_next, SIGNAL(clicked()), this, SLOT(nextTip()));
    setTabOrder(d->m_close, d->m_next);
    setTabOrder(d->m_next, d->m_prev);
    setTabOrder(d->m_prev, d->m_check);
    setTabOrder(d->m_check, d->m_close);
    setFocusProxy(d->m_close);
    retranslateUi();
}

/**
 *  @brief handles user visible string retranslation
 *
 */
void QwwTipWidget::retranslateUi(){
    Q_D(QwwTipWidget);
    d->m_check->setText(tr("Show tips on startup"));
    d->m_prev->setText(tr("&Prev"));
    d->m_next->setText(tr("&Next"));
    d->m_close->setText(tr("&Close"));
}

QwwTipWidget::QwwTipWidget(const QStringList & list, QWidget * parent) : QWidget(parent), priv(new QwwTipWidgetPrivate(this)) {
    Q_D(QwwTipWidget);
    d->m_model = new QStringListModel(list, this);
    d->m_currentTip = d->m_model->index(0,0);
    initUi();
    showTip();
}


QwwTipWidget::~ QwwTipWidget() {
    delete priv;
}

/**
 *  @brief returns index of the current tip
 *
 */
int QwwTipWidget::currentTip() const {
    Q_D(QwwTipWidget);
    return d->m_currentTip.row();
}

/**
 *  @brief initialises widgets and connections
 *
 */
bool QwwTipWidget::tipsEnabled() const {
    Q_D(QwwTipWidget);
    return d->m_check->isChecked();
}

/**
 *  @brief sets \a hw as a header of the widget
 *
 */
void QwwTipWidget::setHeaderWidget(QWidget *hw) {
    Q_D(QwwTipWidget);
    QVBoxLayout *l = dynamic_cast<QVBoxLayout*>(layout());
    if (!l) return;
    delete d->m_headerWidget;
    l->insertWidget(0, hw);
    d->m_headerWidget = hw;
}

/**
 *  @brief returns a pointer to the header widget
 *
 */
QWidget * QwwTipWidget::headerWidget() const {
    Q_D(QwwTipWidget);
    return d->m_headerWidget;
}

/**
 *  @brief returns a pointer to the "next" button
 *
 */
const QPushButton * QwwTipWidget::nextButton() const {
    Q_D(QwwTipWidget);
    return d->m_next;
}

/**
 *  @brief returns a pointer to the "prev" button
 *
 */
const QPushButton * QwwTipWidget::prevButton() const {
    Q_D(QwwTipWidget);
    return d->m_prev;
}

/**
 *  @brief returns a pointer to the "close" button
 *
 */
const QPushButton * QwwTipWidget::closeButton() const {
    Q_D(QwwTipWidget);
    return d->m_close;
}

/**
 *  @brief returns a pointer to the tip canvas
 *
 */
const QTextBrowser * QwwTipWidget::tipCanvas() const {
    Q_D(QwwTipWidget);
    return d->m_browser;
}

bool QwwTipWidget::checkIsVisible() const {
    Q_D(QwwTipWidget);
    return d->m_check->isVisible();
}

bool QwwTipWidget::closeIsVisible() const {
    Q_D(QwwTipWidget);
    return d->m_close->isVisible();
}

void QwwTipWidget::nextTip() {
    Q_D(QwwTipWidget);
    int r = currentTip();
    int c = d->m_model->rowCount();
    r = r==c-1 ? 0 : r+1;
    setCurrentTip(r);
}

void QwwTipWidget::prevTip() {
    Q_D(QwwTipWidget);
    int r = currentTip();
    int c = d->m_model->rowCount();
    r = r==0 ? c-1 : r-1;
    setCurrentTip(r);
}

void QwwTipWidget::setTipsEnabled(bool v) {
    Q_D(QwwTipWidget);
    d->m_check->setChecked(v);
}

void QwwTipWidget::setCheckVisible(bool v) {
    Q_D(QwwTipWidget);
    d->m_check->setVisible(v);
}

void QwwTipWidget::setCloseVisible(bool v){
    Q_D(QwwTipWidget);
    d->m_close->setVisible(v);
}

void QwwTipWidget::setCloseHidden(bool v){
    Q_D(QwwTipWidget);
    d->m_close->setVisible(!v);
}

void QwwTipWidget::setCheckHidden(bool v) {
    Q_D(QwwTipWidget);
    d->m_check->setVisible(!v);
}

void QwwTipWidget::showTip() {
    Q_D(QwwTipWidget);
    d->m_browser->setHtml(d->m_currentTip.data(Qt::DisplayRole).toString());
}

QFrame::Shape QwwTipWidget::canvasFrameShape() const {
    Q_D(QwwTipWidget);
    return d->m_browser->frameShape();
}

void QwwTipWidget::setCanvasFrameShape(QFrame::Shape s) {
    Q_D(QwwTipWidget);
    d->m_browser->setFrameShape(s);
}

void QwwTipWidget::setTips(const QStringList &slist) {
    Q_D(QwwTipWidget);
    delete d->m_model;
    d->m_model = new QStringListModel(slist, this);
    setCurrentTip(0);
}


const QStringList QwwTipWidget::tips() const{
    Q_D(QwwTipWidget);
    return ((QStringListModel*)(d->m_model))->stringList();
}

void QwwTipWidget::setCurrentTip(int v){
    Q_D(QwwTipWidget);
    if(v>=d->m_model->rowCount()) v= d->m_model->rowCount()-1;
    if(v<0) v = 0;
    if(v==currentTip()) return;
    d->m_currentTip = d->m_model->index(v,0);
    showTip();
    emit tipChanged(v);
}

void QwwTipWidget::changeEvent(QEvent *e){
    if(e->type()==QEvent::LanguageChange){
        retranslateUi();
    }
    QWidget::changeEvent(e);
}
